package top.xinsin.interceptor;

import com.alibaba.fastjson2.JSONObject;
import com.auth0.jwt.exceptions.AlgorithmMismatchException;
import com.auth0.jwt.exceptions.SignatureVerificationException;
import com.auth0.jwt.exceptions.TokenExpiredException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.servlet.HandlerInterceptor;
import top.xinsin.util.HttpCodes;
import top.xinsin.util.JWTUtils;
import top.xinsin.util.R;

/**
 * @author xinsin
 * Created On 2023/4/3 15:49
 * @version 1.0
 */
@Slf4j
public class AuthenticationInterceptor implements HandlerInterceptor {
    public static final String OPTIONS = "OPTIONS";
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {

        if (OPTIONS.equals(request.getMethod())) {
            return true;
        }
        // 从http请求头中取出token
        final String token = request.getHeader("Access-Token");
        //解决跨域问题
        response.setHeader("Access-Control-Allow-Origin","*");
        //解决跨域问题
        response.setHeader("Access-Control-Allow-Headers","*");
        R responseData;
        try {
            JWTUtils.verify(token);
            return true;
        } catch (SignatureVerificationException e) {
            log.info("用户验证了无效签名");
            responseData = R.failed(HttpCodes.ACCESS_DENIED,"无效签名");
        }catch (TokenExpiredException e){
            log.info("用户验证的签名已过期");
            responseData = R.failed(HttpCodes.INVALID_TOKEN,"签名已过期");
        }catch (AlgorithmMismatchException e){
            log.info("用户验证的token算法不一致");
            responseData = R.failed(HttpCodes.INVALID_TOKEN,"token算法不一致");
        }catch (Exception e){
            log.info("token无效或者是空的");
            responseData = R.failed(HttpCodes.INVALID_TOKEN,"token无效");
        }
        response.setContentType("application/json;charset=UTF-8");
        response.getWriter().write(JSONObject.toJSONString(responseData));
        return false;
    }
}
